@@ -20,7 +20,6 @@
 
 #include "coeffs/si_gmp.h"
 #include "coeffs/Enumerator.h"
-#include "coeffs/numstats.h" // for STATISTIC(F) counting macro
 
 class CanonicalForm;
 
@@ -131,7 +130,7 @@ struct n_Procs_s
    int     factoryVarOffset;
 
    // general properties:
-   /// TRUE, if nNew/nDelete/nCopy are dummies
+   /// TRUE, if nDelete/nCopy are dummies
    BOOLEAN has_simple_Alloc;
    /// TRUE, if std should make polynomials monic (if nInvers is cheap)
    /// if false, then a gcd routine is used for a content computation
@@ -187,7 +186,7 @@ struct n_Procs_s
    /// convertion to long, 0 if impossible
    long    (*cfInt)(number &n, const coeffs r);
 
-   /// Converts a non-negative number n into a GMP number, 0 if impossible
+   /// Converts a (integer) number n into a GMP number, 0 if impossible
    void     (*cfMPZ)(mpz_t result, number &n, const coeffs r);
 
    /// changes argument  inline: a:= -a
@@ -228,7 +227,7 @@ struct n_Procs_s
            (*cfEqual)(number a,number b, const coeffs r),
            (*cfIsZero)(number a, const coeffs r),
            (*cfIsOne)(number a, const coeffs r),
-       // IsMOne is used for printing os polynomials:
+       // IsMOne is used for printing of polynomials:
        // -1 is only printed for constant monomials
            (*cfIsMOne)(number a, const coeffs r),
        //GreaterZero is used for printing of polynomials:
@@ -378,7 +377,8 @@ struct n_Procs_s
    int     (*cfDivComp)(number a,number b,const coeffs r);
    BOOLEAN (*cfIsUnit)(number a,const coeffs r);
    number  (*cfGetUnit)(number a,const coeffs r);
-   //CF: test if b divides a
+   /// test if b divides a
+   /// cfDivBy(zero,b,r) is true, if b is a zero divisor
    BOOLEAN (*cfDivBy)(number a, number b, const coeffs r);
   /* The following members are for representing the ring Z/n,
      where n is not a prime. We distinguish four cases:
@@ -435,43 +435,39 @@ void nKillChar(coeffs r);
 
 /// initialisations after each ring change
 static FORCE_INLINE void nSetChar(const coeffs r)
-{ STATISTIC(nSetChar);  assume(r!=NULL); assume(r->cfSetChar != NULL); r->cfSetChar(r); }
-
-void           nNew(number * a);
-#define n_New(n, r)           nNew(n)
-
+{ assume(r!=NULL); assume(r->cfSetChar != NULL); r->cfSetChar(r); }
 
 /// Return the characteristic of the coeff. domain.
 static FORCE_INLINE int n_GetChar(const coeffs r)
-{ STATISTIC(n_GetChar); assume(r != NULL); return r->ch; }
+{ assume(r != NULL); return r->ch; }
 
 
 // the access methods (part 2):
 
 /// return a copy of 'n'
 static FORCE_INLINE number n_Copy(number n,    const coeffs r)
-{ STATISTIC(n_Copy);   assume(r != NULL); assume(r->cfCopy!=NULL); return r->cfCopy(n, r); }
+{ assume(r != NULL); assume(r->cfCopy!=NULL); return r->cfCopy(n, r); }
 
 /// delete 'p'
 static FORCE_INLINE void   n_Delete(number* p, const coeffs r)
-{ STATISTIC(n_Delete);   assume(r != NULL); assume(r->cfDelete!= NULL); r->cfDelete(p, r); }
+{ assume(r != NULL); assume(r->cfDelete!= NULL); r->cfDelete(p, r); }
 
 /// TRUE iff 'a' and 'b' represent the same number;
 /// they may have different representations
 static FORCE_INLINE BOOLEAN n_Equal(number a, number b, const coeffs r)
-{ STATISTIC(n_Equal); assume(r != NULL); assume(r->cfEqual!=NULL); return r->cfEqual(a, b, r); }
+{ assume(r != NULL); assume(r->cfEqual!=NULL); return r->cfEqual(a, b, r); }
 
 /// TRUE iff 'n' represents the zero element
 static FORCE_INLINE BOOLEAN n_IsZero(number n, const coeffs r)
-{ STATISTIC(n_IsZero); assume(r != NULL); assume(r->cfIsZero!=NULL); return r->cfIsZero(n,r); }
+{ assume(r != NULL); assume(r->cfIsZero!=NULL); return r->cfIsZero(n,r); }
 
 /// TRUE iff 'n' represents the one element
 static FORCE_INLINE BOOLEAN n_IsOne(number n,  const coeffs r)
-{ STATISTIC(n_IsOne); assume(r != NULL); assume(r->cfIsOne!=NULL); return r->cfIsOne(n,r); }
+{ assume(r != NULL); assume(r->cfIsOne!=NULL); return r->cfIsOne(n,r); }
 
 /// TRUE iff 'n' represents the additive inverse of the one element, i.e. -1
 static FORCE_INLINE BOOLEAN n_IsMOne(number n, const coeffs r)
-{ STATISTIC(n_IsMOne); assume(r != NULL); assume(r->cfIsMOne!=NULL); return r->cfIsMOne(n,r); }
+{ assume(r != NULL); assume(r->cfIsMOne!=NULL); return r->cfIsMOne(n,r); }
 
 /// ordered fields: TRUE iff 'n' is positive;
 /// in Z/pZ: TRUE iff 0 < m <= roundedBelow(p/2), where m is the long
@@ -493,7 +489,7 @@ static FORCE_INLINE BOOLEAN n_IsMOne(number n, const coeffs r)
 ///     start with -
 ///
 static FORCE_INLINE BOOLEAN n_GreaterZero(number n, const coeffs r)
-{ STATISTIC(n_GreaterZero); assume(r != NULL); assume(r->cfGreaterZero!=NULL); return r->cfGreaterZero(n,r); }
+{ assume(r != NULL); assume(r->cfGreaterZero!=NULL); return r->cfGreaterZero(n,r); }
 
 /// ordered fields: TRUE iff 'a' is larger than 'b';
 /// in Z/pZ: TRUE iff la > lb, where la and lb are the long's representing
@@ -510,18 +506,18 @@ static FORCE_INLINE BOOLEAN n_GreaterZero(number n, const coeffs r)
 /// !!! Recommendation: remove implementations for unordered fields
 /// !!!                 and raise errors instead, in these cases
 static FORCE_INLINE BOOLEAN n_Greater(number a, number b, const coeffs r)
-{ STATISTIC(n_Greater); assume(r != NULL); assume(r->cfGreater!=NULL); return r->cfGreater(a,b,r); }
+{ assume(r != NULL); assume(r->cfGreater!=NULL); return r->cfGreater(a,b,r); }
 
 /// TRUE iff n has a multiplicative inverse in the given coeff field/ring r
 static FORCE_INLINE BOOLEAN n_IsUnit(number n, const coeffs r)
-{ STATISTIC(n_IsUnit); assume(r != NULL); assume(r->cfIsUnit!=NULL); return r->cfIsUnit(n,r); }
+{ assume(r != NULL); assume(r->cfIsUnit!=NULL); return r->cfIsUnit(n,r); }
 
 static FORCE_INLINE coeffs n_CoeffRingQuot1(number c, const coeffs r)
-{ STATISTIC(n_CoeffRingQuot1); assume(r != NULL); assume(r->cfQuot1 != NULL); return r->cfQuot1(c, r); }
+{ assume(r != NULL); assume(r->cfQuot1 != NULL); return r->cfQuot1(c, r); }
 
 #ifdef HAVE_RINGS
 static FORCE_INLINE int n_DivComp(number a, number b, const coeffs r)
-{ STATISTIC(n_DivComp); assume(r != NULL); assume(r->cfDivComp!=NULL); return r->cfDivComp (a,b,r); }
+{ assume(r != NULL); assume(r->cfDivComp!=NULL); return r->cfDivComp (a,b,r); }
 
 /// in Z: 1
 /// in Z/kZ (where k is not a prime): largest divisor of n (taken in Z) that
@@ -531,45 +527,45 @@ static FORCE_INLINE int n_DivComp(number a, number b, const coeffs r)
 // CF: shold imply that n/GetUnit(n) is normalized in Z/kZ
 //   it would make more sense to return the inverse...
 static FORCE_INLINE number n_GetUnit(number n, const coeffs r)
-{ STATISTIC(n_GetUnit); assume(r != NULL); assume(r->cfGetUnit!=NULL); return r->cfGetUnit(n,r); }
+{ assume(r != NULL); assume(r->cfGetUnit!=NULL); return r->cfGetUnit(n,r); }
 
 #endif
 
 /// a number representing i in the given coeff field/ring r
 static FORCE_INLINE number n_Init(long i,       const coeffs r)
-{ STATISTIC(n_Init); assume(r != NULL); assume(r->cfInit!=NULL); return r->cfInit(i,r); }
+{ assume(r != NULL); assume(r->cfInit!=NULL); return r->cfInit(i,r); }
 
 /// conversion of a GMP integer to number
 static FORCE_INLINE number n_InitMPZ(mpz_t n,     const coeffs r)
-{ STATISTIC(n_InitMPZ); assume(r != NULL); assume(r->cfInitMPZ != NULL); return r->cfInitMPZ(n,r); }
+{ assume(r != NULL); assume(r->cfInitMPZ != NULL); return r->cfInitMPZ(n,r); }
 
 /// conversion of n to an int; 0 if not possible
 /// in Z/pZ: the representing int lying in (-p/2 .. p/2]
 static FORCE_INLINE long n_Int(number &n,       const coeffs r)
-{ STATISTIC(n_Int); assume(r != NULL); assume(r->cfInt!=NULL); return r->cfInt(n,r); }
+{ assume(r != NULL); assume(r->cfInt!=NULL); return r->cfInt(n,r); }
 
 /// conversion of n to a GMP integer; 0 if not possible
 static FORCE_INLINE void n_MPZ(mpz_t result, number &n,       const coeffs r)
-{ STATISTIC(n_MPZ); assume(r != NULL); assume(r->cfMPZ!=NULL); r->cfMPZ(result, n, r); }
+{ assume(r != NULL); assume(r->cfMPZ!=NULL); r->cfMPZ(result, n, r); }
 
 
 /// in-place negation of n
 /// MUST BE USED: n = n_InpNeg(n) (no copy is returned)
 static FORCE_INLINE number n_InpNeg(number n,     const coeffs r)
-{ STATISTIC(n_InpNeg); assume(r != NULL); assume(r->cfInpNeg!=NULL); return r->cfInpNeg(n,r); }
+{ assume(r != NULL); assume(r->cfInpNeg!=NULL); return r->cfInpNeg(n,r); }
 
 /// return the multiplicative inverse of 'a';
 /// raise an error if 'a' is not invertible
 ///
 /// !!! Recommendation: rename to 'n_Inverse'
 static FORCE_INLINE number n_Invers(number a,  const coeffs r)
-{ STATISTIC(n_Invers); assume(r != NULL); assume(r->cfInvers!=NULL); return r->cfInvers(a,r); }
+{ assume(r != NULL); assume(r->cfInvers!=NULL); return r->cfInvers(a,r); }
 
 /// return a non-negative measure for the complexity of n;
 /// return 0 only when n represents zero;
 /// (used for pivot strategies in matrix computations with entries from r)
 static FORCE_INLINE int    n_Size(number n,    const coeffs r)
-{ STATISTIC(n_Size); assume(r != NULL); assume(r->cfSize!=NULL); return r->cfSize(n,r); }
+{ assume(r != NULL); assume(r->cfSize!=NULL); return r->cfSize(n,r); }
 
 /// inplace-normalization of n;
 /// produces some canonical representation of n;
@@ -577,106 +573,84 @@ static FORCE_INLINE int    n_Size(number n,    const coeffs r)
 /// !!! Recommendation: remove this method from the user-interface, i.e.,
 /// !!!                 this should be hidden
 static FORCE_INLINE void   n_Normalize(number& n, const coeffs r)
-{ STATISTIC(n_Normalize); assume(r != NULL); assume(r->cfNormalize!=NULL); r->cfNormalize(n,r); }
+{ assume(r != NULL); assume(r->cfNormalize!=NULL); r->cfNormalize(n,r); }
 
 /// write to the output buffer of the currently used reporter
 //CF: the "&" should be removed, as one wants to write constants as well
 static FORCE_INLINE void   n_WriteLong(number n,  const coeffs r)
-{ STATISTIC(n_WriteLong); assume(r != NULL); assume(r->cfWriteLong!=NULL); r->cfWriteLong(n,r); }
+{ assume(r != NULL); assume(r->cfWriteLong!=NULL); r->cfWriteLong(n,r); }
 
 /// write to the output buffer of the currently used reporter
 /// in a shortest possible way, e.g. in K(a): a2 instead of a^2
 static FORCE_INLINE void   n_WriteShort(number n,  const coeffs r)
-{ STATISTIC(n_WriteShort); assume(r != NULL); assume(r->cfWriteShort!=NULL); r->cfWriteShort(n,r); }
+{ assume(r != NULL); assume(r->cfWriteShort!=NULL); r->cfWriteShort(n,r); }
 
 static FORCE_INLINE void   n_Write(number n,  const coeffs r, const BOOLEAN bShortOut = TRUE)
-{ STATISTIC(n_Write); if (bShortOut) n_WriteShort(n, r); else n_WriteLong(n, r); }
+{ if (bShortOut) n_WriteShort(n, r); else n_WriteLong(n, r); }
 
 
 /// !!! Recommendation: This method is too cryptic to be part of the user-
 /// !!!                 interface. As defined here, it is merely a helper
 /// !!!                 method for parsing number input strings.
 static FORCE_INLINE const char *n_Read(const char * s, number * a, const coeffs r)
-{ STATISTIC(n_Read); assume(r != NULL); assume(r->cfRead!=NULL); return r->cfRead(s, a, r); }
+{ assume(r != NULL); assume(r->cfRead!=NULL); return r->cfRead(s, a, r); }
 
 /// return the denominator of n
 /// (if elements of r are by nature not fractional, result is 1)
 static FORCE_INLINE number n_GetDenom(number& n, const coeffs r)
-{ STATISTIC(n_GetDenom); assume(r != NULL); assume(r->cfGetDenom!=NULL); return r->cfGetDenom(n, r); }
+{ assume(r != NULL); assume(r->cfGetDenom!=NULL); return r->cfGetDenom(n, r); }
 
 /// return the numerator of n
 /// (if elements of r are by nature not fractional, result is n)
 static FORCE_INLINE number n_GetNumerator(number& n, const coeffs r)
-{ STATISTIC(n_GetNumerator); assume(r != NULL); assume(r->cfGetNumerator!=NULL); return r->cfGetNumerator(n, r); }
+{ assume(r != NULL); assume(r->cfGetNumerator!=NULL); return r->cfGetNumerator(n, r); }
 
 /// return the quotient of 'a' and 'b', i.e., a/b;
 /// raises an error if 'b' is not invertible in r
 /// exception in Z: raises an error if 'a' is not divisible by 'b'
 /// always: n_Div(a,b,r)*b+n_IntMod(a,b,r)==a
 static FORCE_INLINE number n_Div(number a, number b, const coeffs r)
-{ STATISTIC(n_Div); assume(r != NULL); assume(r->cfDiv!=NULL); return r->cfDiv(a,b,r); }
+{ assume(r != NULL); assume(r->cfDiv!=NULL); return r->cfDiv(a,b,r); }
 
 /// assume that there is a canonical subring in cf and we know
 /// that division is possible for these a and b in the subring,
 /// n_ExactDiv performs it, may skip additional tests.
 /// Can always be substituted by n_Div at the cost of larger  computing time.
 static FORCE_INLINE number n_ExactDiv(number a, number b, const coeffs r)
-{ STATISTIC(n_ExactDiv); assume(r != NULL); assume(r->cfExactDiv!=NULL); return r->cfExactDiv(a,b,r); }
+{ assume(r != NULL); assume(r->cfExactDiv!=NULL); return r->cfExactDiv(a,b,r); }
 
 /// for r a field, return n_Init(0,r)
 /// always: n_Div(a,b,r)*b+n_IntMod(a,b,r)==a
 /// n_IntMod(a,b,r) >=0
 static FORCE_INLINE number n_IntMod(number a, number b, const coeffs r)
-{ STATISTIC(n_IntMod); assume(r != NULL); return r->cfIntMod(a,b,r); }
+{ assume(r != NULL); return r->cfIntMod(a,b,r); }
 
 /// fill res with the power a^b
 static FORCE_INLINE void   n_Power(number a, int b, number *res, const coeffs r)
-{ STATISTIC(n_Power); assume(r != NULL); assume(r->cfPower!=NULL); r->cfPower(a,b,res,r); }
+{ assume(r != NULL); assume(r->cfPower!=NULL); r->cfPower(a,b,res,r); }
 
 /// return the product of 'a' and 'b', i.e., a*b
 static FORCE_INLINE number n_Mult(number a, number b, const coeffs r)
-{ STATISTIC(n_Mult); assume(r != NULL); assume(r->cfMult!=NULL); return r->cfMult(a, b, r); }
+{ assume(r != NULL); assume(r->cfMult!=NULL); return r->cfMult(a, b, r); }
 
 /// multiplication of 'a' and 'b';
 /// replacement of 'a' by the product a*b
 static FORCE_INLINE void n_InpMult(number &a, number b, const coeffs r)
-{ STATISTIC(n_InpMult); assume(r != NULL); assume(r->cfInpMult!=NULL); r->cfInpMult(a,b,r); }
+{ assume(r != NULL); assume(r->cfInpMult!=NULL); r->cfInpMult(a,b,r); }
 
 /// addition of 'a' and 'b';
 /// replacement of 'a' by the sum a+b
 static FORCE_INLINE void n_InpAdd(number &a, number b, const coeffs r)
-{ STATISTIC(n_InpAdd); assume(r != NULL); assume(r->cfInpAdd!=NULL); r->cfInpAdd(a,b,r);
-
-#ifdef HAVE_NUMSTATS
-  // avoid double counting
-  if( r->cfIsZero(a,r) ) STATISTIC(n_CancelOut);
-#endif
-}
+{ assume(r != NULL); assume(r->cfInpAdd!=NULL); r->cfInpAdd(a,b,r); }
 
 /// return the sum of 'a' and 'b', i.e., a+b
 static FORCE_INLINE number n_Add(number a, number b, const coeffs r)
-{ STATISTIC(n_Add); assume(r != NULL); assume(r->cfAdd!=NULL); const number sum = r->cfAdd(a, b, r);
-
-#ifdef HAVE_NUMSTATS
-  // avoid double counting
-  if( r->cfIsZero(sum,r) ) STATISTIC(n_CancelOut);
-#endif
-
- return sum;
-}
+{ assume(r != NULL); assume(r->cfAdd!=NULL); return r->cfAdd(a, b, r); }
 
 
 /// return the difference of 'a' and 'b', i.e., a-b
 static FORCE_INLINE number n_Sub(number a, number b, const coeffs r)
-{ STATISTIC(n_Sub); assume(r != NULL); assume(r->cfSub!=NULL); const number d = r->cfSub(a, b, r);
-
-#ifdef HAVE_NUMSTATS
-  // avoid double counting
-  if( r->cfIsZero(d,r) ) STATISTIC(n_CancelOut);
-#endif
-
-  return d;
-}
+{ assume(r != NULL); assume(r->cfSub!=NULL); return r->cfSub(a, b, r); }
 
 /// in Z: return the gcd of 'a' and 'b'
 /// in Z/nZ, Z/2^kZ: computed as in the case Z
@@ -685,24 +659,24 @@ static FORCE_INLINE number n_Sub(number a, number b, const coeffs r)
 /// in K(a)/<p(a)>: not implemented
 /// in K(t_1, ..., t_n): not implemented
 static FORCE_INLINE number n_Gcd(number a, number b, const coeffs r)
-{ STATISTIC(n_Gcd); assume(r != NULL); assume(r->cfGcd!=NULL); return r->cfGcd(a,b,r); }
+{ assume(r != NULL); assume(r->cfGcd!=NULL); return r->cfGcd(a,b,r); }
 static FORCE_INLINE number n_SubringGcd(number a, number b, const coeffs r)
-{ STATISTIC(n_SubringGcd); assume(r != NULL); assume(r->cfSubringGcd!=NULL); return r->cfSubringGcd(a,b,r); }
+{ assume(r != NULL); assume(r->cfSubringGcd!=NULL); return r->cfSubringGcd(a,b,r); }
 
 /// beware that ExtGCD is only relevant for a few chosen coeff. domains
 /// and may perform something unexpected in some cases...
 static FORCE_INLINE number n_ExtGcd(number a, number b, number *s, number *t, const coeffs r)
-{ STATISTIC(n_ExtGcd); assume(r != NULL); assume(r->cfExtGcd!=NULL); return r->cfExtGcd (a,b,s,t,r); }
+{ assume(r != NULL); assume(r->cfExtGcd!=NULL); return r->cfExtGcd (a,b,s,t,r); }
 static FORCE_INLINE number n_XExtGcd(number a, number b, number *s, number *t, number *u, number *v, const coeffs r)
-{ STATISTIC(n_XExtGcd); assume(r != NULL); assume(r->cfXExtGcd!=NULL); return r->cfXExtGcd (a,b,s,t,u,v,r); }
+{ assume(r != NULL); assume(r->cfXExtGcd!=NULL); return r->cfXExtGcd (a,b,s,t,u,v,r); }
 static FORCE_INLINE number  n_EucNorm(number a, const coeffs r)
-{ STATISTIC(n_EucNorm); assume(r != NULL); assume(r->cfEucNorm!=NULL); return r->cfEucNorm (a,r); }
+{ assume(r != NULL); assume(r->cfEucNorm!=NULL); return r->cfEucNorm (a,r); }
 /// if r is a ring with zero divisors, return an annihilator!=0 of b
 /// otherwise return NULL
 static FORCE_INLINE number  n_Ann(number a, const coeffs r)
-{ STATISTIC(n_Ann); assume(r != NULL); return r->cfAnn (a,r); }
+{ assume(r != NULL); return r->cfAnn (a,r); }
 static FORCE_INLINE number  n_QuotRem(number a, number b, number *q, const coeffs r)
-{ STATISTIC(n_QuotRem); assume(r != NULL); assume(r->cfQuotRem!=NULL); return r->cfQuotRem (a,b,q,r); }
+{ assume(r != NULL); assume(r->cfQuotRem!=NULL); return r->cfQuotRem (a,b,q,r); }
 
 
 /// in Z: return the lcm of 'a' and 'b'
@@ -711,18 +685,17 @@ static FORCE_INLINE number  n_QuotRem(number a, number b, number *q, const coeff
 /// in K(a)/<p(a)>: not implemented
 /// in K(t_1, ..., t_n): not implemented
 static FORCE_INLINE number n_Lcm(number a, number b, const coeffs r)
-{ STATISTIC(n_Lcm); assume(r != NULL); assume(r->cfLcm!=NULL); return r->cfLcm(a,b,r); }
+{ assume(r != NULL); assume(r->cfLcm!=NULL); return r->cfLcm(a,b,r); }
 
 /// assume that r is a quotient field (otherwise, return 1)
 /// for arguments (a1/a2,b1/b2) return (lcm(a1,b2)/1)
 static FORCE_INLINE number n_NormalizeHelper(number a, number b, const coeffs r)
-{ STATISTIC(n_NormalizeHelper); assume(r != NULL); assume(r->cfNormalizeHelper!=NULL); return r->cfNormalizeHelper(a,b,r); }
+{ assume(r != NULL); assume(r->cfNormalizeHelper!=NULL); return r->cfNormalizeHelper(a,b,r); }
 
 number ndCopyMap(number a, const coeffs src, const coeffs dst);
 /// set the mapping function pointers for translating numbers from src to dst
 static FORCE_INLINE nMapFunc n_SetMap(const coeffs src, const coeffs dst)
-{ STATISTIC(n_SetMap);
-  assume(src != NULL && dst != NULL); assume(dst->cfSetMap!=NULL);
+{ assume(src != NULL && dst != NULL); assume(dst->cfSetMap!=NULL);
   if (src==dst) return ndCopyMap;
   return dst->cfSetMap(src,dst);
 }
@@ -731,7 +704,7 @@ static FORCE_INLINE nMapFunc n_SetMap(const coeffs src, const coeffs dst)
 /// test whether n is a correct number;
 /// only used if LDEBUG is defined
 static FORCE_INLINE BOOLEAN n_DBTest(number n, const char *filename, const int linenumber, const coeffs r)
-{ STATISTIC(n_Test); assume(r != NULL); assume(r->cfDBTest != NULL); return r->cfDBTest(n, filename, linenumber, r); }
+{ assume(r != NULL); assume(r->cfDBTest != NULL); return r->cfDBTest(n, filename, linenumber, r); }
 /// BOOLEAN n_Test(number a, const coeffs r)
 #define n_Test(a,r)  n_DBTest(a, __FILE__, __LINE__, r)
 #else
@@ -741,7 +714,7 @@ static FORCE_INLINE BOOLEAN n_DBTest(number n, const char *filename, const int l
 
 /// output the coeff description
 static FORCE_INLINE void   n_CoeffWrite(const coeffs r, BOOLEAN details = TRUE)
-{ STATISTIC(n_CoeffWrite); assume(r != NULL); assume(r->cfCoeffWrite != NULL); r->cfCoeffWrite(r, details); }
+{ assume(r != NULL); assume(r->cfCoeffWrite != NULL); r->cfCoeffWrite(r, details); }
 
 // Tests:
 #ifdef HAVE_RINGS
@@ -775,7 +748,7 @@ static FORCE_INLINE BOOLEAN nCoeff_is_Domain(const coeffs r)
 /// in Z/2^kZ: TRUE iff ((a = 0 mod 2^k) and (b = 0 or b is a power of 2))
 ///                  or ((a, b <> 0) and (b/gcd(a, b) is odd))
 static FORCE_INLINE BOOLEAN n_DivBy(number a, number b, const coeffs r)
-{ STATISTIC(n_DivBy); assume(r != NULL);
+{ assume(r != NULL);
 #ifdef HAVE_RINGS
   if( nCoeff_is_Ring(r) )
   {
@@ -786,21 +759,21 @@ static FORCE_INLINE BOOLEAN n_DivBy(number a, number b, const coeffs r)
 }
 
 static FORCE_INLINE number n_ChineseRemainderSym(number *a, number *b, int rl, BOOLEAN sym,CFArray &inv_cache,const coeffs r)
-{ STATISTIC(n_ChineseRemainderSym); assume(r != NULL); assume(r->cfChineseRemainder != NULL); return r->cfChineseRemainder(a,b,rl,sym,inv_cache,r); }
+{ assume(r != NULL); assume(r->cfChineseRemainder != NULL); return r->cfChineseRemainder(a,b,rl,sym,inv_cache,r); }
 
 static FORCE_INLINE number n_Farey(number a, number b, const coeffs r)
-{ STATISTIC(n_Farey); assume(r != NULL); assume(r->cfFarey != NULL); return r->cfFarey(a,b,r); }
+{ assume(r != NULL); assume(r->cfFarey != NULL); return r->cfFarey(a,b,r); }
 
 static FORCE_INLINE int n_ParDeg(number n, const coeffs r)
-{ STATISTIC(n_ParDeg); assume(r != NULL); assume(r->cfParDeg != NULL); return r->cfParDeg(n,r); }
+{ assume(r != NULL); assume(r->cfParDeg != NULL); return r->cfParDeg(n,r); }
 
 /// Returns the number of parameters
 static FORCE_INLINE int n_NumberOfParameters(const coeffs r)
-{ STATISTIC(n_NumberOfParameters); assume(r != NULL); return r->iNumberOfParameters; }
+{ assume(r != NULL); return r->iNumberOfParameters; }
 
 /// Returns a (const!) pointer to (const char*) names of parameters
 static FORCE_INLINE char const * * n_ParameterNames(const coeffs r)
-{ STATISTIC(n_ParameterNames); assume(r != NULL); return r->pParameterNames; }
+{ assume(r != NULL); return r->pParameterNames; }
 
 /// return the (iParameter^th) parameter as a NEW number
 /// NOTE: parameter numbering: 1..n_NumberOfParameters(...)
@@ -808,14 +781,14 @@ static FORCE_INLINE number n_Param(const int iParameter, const coeffs r)
 { assume(r != NULL);
   assume((iParameter >= 1) || (iParameter <= n_NumberOfParameters(r)));
   assume(r->cfParameter != NULL);
-  STATISTIC(n_Param); return r->cfParameter(iParameter, r);
+  return r->cfParameter(iParameter, r);
 }
 
 static FORCE_INLINE number  n_RePart(number i, const coeffs cf)
-{ STATISTIC(n_RePart); assume(cf != NULL); assume(cf->cfRePart!=NULL); return cf->cfRePart(i,cf); }
+{ assume(cf != NULL); assume(cf->cfRePart!=NULL); return cf->cfRePart(i,cf); }
 
 static FORCE_INLINE number  n_ImPart(number i, const coeffs cf)
-{ STATISTIC(n_ImPart); assume(cf != NULL); assume(cf->cfImPart!=NULL); return cf->cfImPart(i,cf); }
+{ assume(cf != NULL); assume(cf->cfImPart!=NULL); return cf->cfImPart(i,cf); }
 
 /// returns TRUE, if r is not a field and r has non-trivial units
 static FORCE_INLINE BOOLEAN nCoeff_has_Units(const coeffs r)
@@ -926,7 +899,7 @@ static FORCE_INLINE BOOLEAN nCoeff_is_CF(const coeffs r)
 static FORCE_INLINE BOOLEAN nCoeff_has_simple_inverse(const coeffs r)
 { assume(r != NULL); return r->has_simple_Inverse; }
 
-/// TRUE if n_Delete/n_New are empty operations
+/// TRUE if n_Delete is empty operation
 static FORCE_INLINE BOOLEAN nCoeff_has_simple_Alloc(const coeffs r)
 { assume(r != NULL); return r->has_simple_Alloc; }
 
@@ -950,14 +923,14 @@ static FORCE_INLINE BOOLEAN nCoeff_is_transExt(const coeffs r)
 /// NOTE/TODO: see also the description by Hans
 /// TODO: rename into n_ClearIntegerContent
 static FORCE_INLINE void n_ClearContent(ICoeffsEnumerator& numberCollectionEnumerator, number& c, const coeffs r)
-{ STATISTIC(n_ClearContent); assume(r != NULL); r->cfClearContent(numberCollectionEnumerator, c, r); }
+{ assume(r != NULL); r->cfClearContent(numberCollectionEnumerator, c, r); }
 
 /// (inplace) Clears denominators on a collection of numbers
 /// number @em d is the LCM of all the coefficient denominators (i.e. the number
 /// with which all the number coeffs. were multiplied)
 /// NOTE/TODO: see also the description by Hans
 static FORCE_INLINE void n_ClearDenominators(ICoeffsEnumerator& numberCollectionEnumerator, number& d, const coeffs r)
-{ STATISTIC(n_ClearDenominators); assume(r != NULL); r->cfClearDenominators(numberCollectionEnumerator, d, r); }
+{ assume(r != NULL); r->cfClearDenominators(numberCollectionEnumerator, d, r); }
 
 // convenience helpers (no number returned - but the input enumeration
 // is to be changed
@@ -966,10 +939,10 @@ static FORCE_INLINE void n_ClearDenominators(ICoeffsEnumerator& numberCollection
 // *p_Content) and p_Cleardenom_n (which doesn't)!!!
 
 static FORCE_INLINE void n_ClearContent(ICoeffsEnumerator& numberCollectionEnumerator, const coeffs r)
-{ STATISTIC(n_ClearContent); number c; n_ClearContent(numberCollectionEnumerator, c, r); n_Delete(&c, r); }
+{ number c; n_ClearContent(numberCollectionEnumerator, c, r); n_Delete(&c, r); }
 
 static FORCE_INLINE void n_ClearDenominators(ICoeffsEnumerator& numberCollectionEnumerator, const coeffs r)
-{ STATISTIC(n_ClearDenominators); assume(r != NULL); number d; n_ClearDenominators(numberCollectionEnumerator, d, r); n_Delete(&d, r); }
+{ assume(r != NULL); number d; n_ClearDenominators(numberCollectionEnumerator, d, r); n_Delete(&d, r); }
 
 
 /// print a number (BEWARE of string buffers!)
@@ -981,29 +954,29 @@ void   n_Print(number& a,  const coeffs r);
 /// TODO: make it a virtual method of coeffs, together with:
 /// Decompose & Compose, rParameter & rPar
 static FORCE_INLINE char * nCoeffString(const coeffs cf)
-{ STATISTIC(nCoeffString); assume( cf != NULL ); return cf->cfCoeffString(cf); }
+{ assume( cf != NULL ); return cf->cfCoeffString(cf); }
 
 
 static FORCE_INLINE char * nCoeffName (const coeffs cf)
-{ STATISTIC(nCoeffName); assume( cf != NULL ); return cf->cfCoeffName(cf); }
+{ assume( cf != NULL ); return cf->cfCoeffName(cf); }
 
 static FORCE_INLINE number n_Random(siRandProc p, number p1, number p2, const coeffs cf)
-{ STATISTIC(n_Random); assume( cf != NULL ); assume( cf->cfRandom != NULL );  return cf->cfRandom(p, p1, p2, cf); }
+{ assume( cf != NULL ); assume( cf->cfRandom != NULL );  return cf->cfRandom(p, p1, p2, cf); }
 
 /// io via ssi:
 static FORCE_INLINE void n_WriteFd(number a, const ssiInfo *f, const coeffs r)
-{ STATISTIC(n_WriteFd); assume(r != NULL); assume(r->cfWriteFd != NULL); return r->cfWriteFd(a, f, r); }
+{ assume(r != NULL); assume(r->cfWriteFd != NULL); return r->cfWriteFd(a, f, r); }
 
 /// io via ssi:
 static FORCE_INLINE number n_ReadFd( const ssiInfo *f, const coeffs r)
-{ STATISTIC(n_ReadFd); assume(r != NULL); assume(r->cfReadFd != NULL); return r->cfReadFd(f, r); }
+{ assume(r != NULL); assume(r->cfReadFd != NULL); return r->cfReadFd(f, r); }
 
 
-// the following wrappers went to numbers.cc since they needed factory
-// knowledge!
-number n_convFactoryNSingN( const CanonicalForm n, const coeffs r);
+static FORCE_INLINE number n_convFactoryNSingN( const CanonicalForm n, const coeffs r)
+{ assume(r != NULL); assume(r->convFactoryNSingN != NULL); return r->convFactoryNSingN(n, r); }
 
-CanonicalForm n_convSingNFactoryN( number n, BOOLEAN setChar, const coeffs r );
+static FORCE_INLINE CanonicalForm n_convSingNFactoryN( number n, BOOLEAN setChar, const coeffs r )
+{ assume(r != NULL); assume(r->convSingNFactoryN != NULL); return r->convSingNFactoryN(n, setChar, r); }
 
 
 // TODO: remove the following functions...
