@@ -9,14 +9,17 @@
 #define MOD2_H
 
 /* please include singularconfig.h exclusively via <kernel/mod2.h> and before any other header */
-# include <singularconfig.h>
-# include <misc/auxiliary.h>
+# include "singularconfig.h"
+# include "misc/auxiliary.h"
+
+#if defined(HAVE_MPFR_H) && defined(HAVE_LIBMPFR)
+#define HAVE_SVD
+#endif
 
-#undef VERSION
-#define VERSION "4.1.0"
 #define SINGULAR_MAJOR_VERSION 4
-#define SINGULAR_MINOR_VERSION 1
-#define SINGULAR_SUB_VERSION 0
+#define SINGULAR_MINOR_VERSION 3
+#define SINGULAR_SUB_VERSION 2
+#define SINGULAR_PATCHLEVEL 4
 #define S_ROOT_DIR ""
 
 /*******************************************************************
@@ -50,20 +53,19 @@
 /* linear algebra extensions from pcv.h/pcv.cc */
 #define HAVE_PCV 1
 
-/* procedures to compute groebner bases with the f5 implementation */
-/* still testing */
-#undef HAVE_F5
-
-/* procedures to compute groebner bases with the f5c implementation */
-/* still testing */
-#undef HAVE_F5C
-
 /* procedures to compute with units */
 #define HAVE_UNITS
 
 /* Define to use scanner when loading libraries */
 #define HAVE_LIBPARSER
 
+/* define for parallel processes with shared memory */
+#ifndef __CCYGWIN__
+#if !defined(__MACH__) || !defined(__APPLE__)
+#define HAVE_VSPACE 1
+#endif
+#endif
+
 /*#define PROFILING*/
 #ifdef PROFILING
 #define PROFILER ,0,0
@@ -82,11 +84,10 @@
 #define ALIGN_8
 #endif
 
-#define SINGULAR_PATCHLEVEL 3
 #define SINGULAR_VERSION ((SINGULAR_MAJOR_VERSION*1000 + SINGULAR_MINOR_VERSION*100 + SINGULAR_SUB_VERSION*10)+SINGULAR_PATCHLEVEL)
 
 /*******************************************************************
- * Miscellanous Defines
+ * Miscellaneous Defines
  ******************************************************************/
 #ifndef HAVE_LIBPARSER
 #  undef YYLPDEBUG
@@ -114,7 +115,15 @@
 #define SEEK_SET 0
 #endif
 
-#define HALT() m2_end(2)
+#ifdef __cplusplus
+extern "C" {
+#endif
+void  m2_end(int i);
+#ifdef __cplusplus
+}
+#endif
+
+static inline void HALT(void) { m2_end(2);}
 
 /* define OLD_RES for res/sres/mres(i,j,k) */
 #undef OLD_RES
@@ -123,17 +132,6 @@
 /* #define MAX_INT_LEN 11 */
 
 
-#ifdef DO_PROFILE
-/* define to enable explicit profiling of some crucial inline
- * routines and defines  */
-#undef DO_DEEP_PROFILE
-#endif
-
-/* define to enable assume */
-#ifndef HAVE_ASSUME
-#undef HAVE_ASSUME
-#endif
-
 /* define LINKAGE to "extern C" if compiling for shared libs */
 #ifndef LINKAGE
 #if defined(PIC)
@@ -154,9 +152,6 @@
 /* undefine to enable inline */
 #define NO_INLINE
 
-/* undefine to disable assume -- should normally be defined for SING_NDEBUG */
-#define HAVE_ASSUME
-
 /* undef PDEBUG to disable checks of polys
 
  define PDEBUG to
@@ -190,13 +185,13 @@
 
    The omDebug routines are controlled by the values of OM_TRACK, OM_CHECK
    and OM_KEEP.  There meaning is roughly as follows:
-   OM_TRACK: strored with address                              : extra space
-     0     : no additional info is stored                      : 0
-     1     : file:line of location where address was allocated : 1 word
-     2     : plus backtrace of stack where adress was allocated: 6 words
-     3     : plus size/bin info and front-, and back padding   : 9 words
-     4     : plus file:line of location where adress was freed : 10 words
-     5     : plus backtrace of stack where adress was allocated: 15 words
+   OM_TRACK: stored with address                                 : extra space
+     0     : no additional info is stored                        : 0
+     1     : file:line of location where address was allocated   : 1 word
+     2     : plus backtrace of stack where address was allocated : 6 words
+     3     : plus size/bin info and front-, and back padding     : 9 words
+     4     : plus file:line of location where address was freed  : 10 words
+     5     : plus backtrace of stack where address was allocated : 15 words
    OM_CHECK: checks done
      0     : no checks
      1     : constant-time checks: i.e. addr checks only
@@ -215,7 +210,7 @@
      #define OM_TRACK 5
      #define OM_KEEP  1
      #include "mod2.h"
-     #include <omalloc/omalloc.h>
+     #include "omalloc/omalloc.h"
    ensures that all memory allocs/free in this file are done with
    OM_CHECK==3 and OM_TRACK==5, and that all addresses allocated/freed
    in this file are only marked as free and never really freed.
@@ -228,7 +223,7 @@
      om_Opts.MinCheck = 3; om_Opts.MinTrack = 5; omOpts.Keep = LONG_MAX;
      ExternalRoutine();
      om_Opts.MinCheck = check; omOpts.MinTrack = track; omOpts.Keep = keep;
-   ensures that all calls omDebug routines  occuring during the computation of
+   ensures that all calls omDebug routines  occurring during the computation of
    ExternalRoutine() are done with OM_CHECK==3 and OM_TRACK==5, and
    calls to omFree only mark addresses as free and not really free them.
 
@@ -243,7 +238,7 @@
        #define OM_TRACK 5
        #define OM_KEEP  1
        #include "mod2.h"
-       #include <omalloc/omalloc.h>
+       #include "omalloc/omalloc.h"
      Under dynamic scope, do (e.g., from within the debugger):
        om_Opts.MinCheck = 3; om_Opts.MinTrack = 5; omOpts.Keep = LONG_MAX;
    + to find out where "memory corruption" occurred, increase value of
@@ -258,7 +253,7 @@
      NULL pointers and sizes == 0
    + You can safely use any free/realloc routine in combination with any alloc
      routine (including the debug versions): E.g., an address allocated with
-     omAllocBin can be freed with omfree, or an adress allocated with
+     omAllocBin can be freed with omfree, or an address allocated with
      om(Debug)Alloc can be freed with omfree, or omFree, or omFreeSize, etc.
      However, keep in mind that the efficiency decreases from
      Bin over Size to General routines (i.e., omFreeBin is more efficient than
@@ -378,15 +373,15 @@ extern "C" {
 extern int dReportError(const char* fmt, ...);
 /* within a debugger, set a breakpoint on dErrorBreak
 * which is called after the error has been reported */
-extern void dErrorBreak();
+extern void dErrorBreak(void);
 #ifdef __cplusplus
 }
 #endif
 
-#ifndef HAVE_ASSUME
+#ifdef SING_NDEBUG
 #define assume(x) do {} while (0)
 #define r_assume(x) do {} while (0)
-#else /* ! HAVE_ASSUME */
+#else /* SING_NDEBUG */
 
 #define assume_violation(s,f,l) \
   dReportError("assume violation at %s:%d condition: %s", f,l,s)
@@ -414,10 +409,10 @@ do                                              \
   }                                             \
 }                                               \
 while (0)
-#endif /* HAVE_ASSUME */
+#endif /* SING_NDEBUG */
 
 /* do have RDEBUG, unless we are doing the very real thing */
-#ifdef HAVE_ASSUME
+#ifndef SING_NDEBUG
 #ifndef RDEBUG
 #define RDEBUG
 #endif
